"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var parser = __importStar(require("@babel/parser"));
var traverse_1 = __importDefault(require("@babel/traverse"));
function checkEsnextSyntax(source, visitorsReturnSyntaxDescription) {
    var ret = {};
    var setDesc = function (desc) {
        ret[desc] = (ret[desc] || 0) + 1;
    };
    var ast = parser.parse(source, { sourceType: 'module' });
    var visitors = {};
    if (visitorsReturnSyntaxDescription) {
        visitors = Object.keys(visitorsReturnSyntaxDescription).reduce(function (obj, key) {
            var func = visitorsReturnSyntaxDescription[key];
            obj[key] = function () {
                var arg = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    arg[_i] = arguments[_i];
                }
                // @ts-ignore
                var syntaxDesc = func.apply(void 0, arg);
                if (syntaxDesc) {
                    setDesc("custom: " + syntaxDesc);
                }
            };
            return obj;
        }, {});
    }
    traverse_1.default(ast, __assign(__assign({}, visitors), { ImportDeclaration: function (path) {
            setDesc('import statement');
        },
        Import: function (path) {
            setDesc('import()');
        },
        ExportDeclaration: function (path) {
            setDesc('export statement');
        },
        ArrowFunctionExpression: function (path) {
            setDesc('arrow function');
        },
        FunctionDeclaration: function (path) {
            if (path.node.generator) {
                setDesc('generator function');
            }
            else if (path.node.async) {
                setDesc('async function');
            }
        },
        FunctionExpression: function (path) {
            if (path.node.generator) {
                setDesc('generator function');
            }
            else if (path.node.async) {
                setDesc('async function');
            }
        },
        ObjectMethod: function (path) {
            if (path.node.generator) {
                setDesc('generator function');
            }
            else if (path.node.async) {
                setDesc('async function');
            }
        },
        ObjectProperty: function (path) {
            if (path.node.computed) {
                setDesc("computed syntax like: { ['someKey']: 1 }");
            }
            if (path.node.shorthand) {
                setDesc("object property shorthand");
            }
        },
        TaggedTemplateExpression: function (path) {
            setDesc("TaggedTemplate: foo`arg`");
        },
        TemplateLiteral: function (path) {
            setDesc("template string");
        },
        Super: function (path) {
            setDesc("'Super' keyword");
        },
        Class: function (path) {
            setDesc("class");
        },
        ClassProperty: function (path) {
            setDesc(path.node.static ? 'class static property' : 'class property');
        },
        ClassMethod: function (path) {
            setDesc(path.node.static ? 'class static method' : 'class method');
        },
        ClassPrivateMethod: function (path) {
            setDesc(path.node.static ? 'class private static method' : 'class private method');
        },
        ClassPrivateProperty: function (path) {
            setDesc(path.node.static ? 'class private static property' : 'class private property');
        },
        ForOfStatement: function (path) {
            setDesc(path.node.await ? "for await of" : 'for of');
        },
        ObjectPattern: function (path) {
            setDesc("destructuring like: const { b } = {};");
        },
        ArrayPattern: function (path) {
            setDesc("destructuring like: const [a] = []; function f ([c]) {}");
        },
        RestElement: function (path) {
            setDesc("rest syntax");
        },
        SpreadElement: function (path) {
            setDesc("spread syntax");
        },
        VariableDeclaration: function (path) {
            if (path.node.kind === 'let' || path.node.kind === 'const') {
                setDesc('let or const');
            }
        },
        AssignmentPattern: function (path) {
            setDesc("assignment like: const [a = 1] = []; const { b = 1 } = {}; function f (c = 1) {}");
        },
        BinaryExpression: function (path) {
            if (path.node.operator === '**') {
                setDesc('power operator: **');
            }
        },
        LogicalExpression: function (path) {
            if (path.node.operator === '??') {
                setDesc('logical operator: ??');
            }
        },
        AwaitExpression: function (path) {
            setDesc('await expression');
        } }));
    return ret;
}
exports.checkEsnextSyntax = checkEsnextSyntax;
